/*
 * 3pbmtopcm.c:
 *
 * Copyright (c) 2001 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] =
  "$Id: 3pbmtopcm.c,v 1.3 2001/05/25 01:24:37 root Exp $";

/*
 * $Log: 3pbmtopcm.c,v $
 * Revision 1.3  2001/05/25 01:24:37  root
 * #
 *
 * Revision 1.2  2001/05/25 01:24:15  root
 * #
 *
 */


#include <malloc.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>

static int
getfdline (int fd, char *buf, int buflen)
{
  char *ptr = buf;

  buflen--;

  while (buflen > 0)
    {

      read (fd, ptr, 1);
      *(ptr + 1) = 0;
      buflen--;

      if (*ptr == '\n')
        {
          if (*buf == '#')
            {
              ptr = buf;
            }
          else
            {
              return 1;
            }
        }
      else
        ptr++;
    }

  return 0;
}


int
main (int argc, char *argv[])
{
  int infd, outfd;
  off_t inoff, outoff;
  size_t inlength, outlength;
  int w, h, ibpl;
  int c, q;
  char *ext[3] = { "red", "green", "blue" };
  int pass;

  unsigned char *buf;
  unsigned char *inbasename = NULL;
  char header[1024];
  char inname[1024];

  infd = 0;
  outfd = 1;
  while ((c = getopt (argc, argv, "i:o:")) != EOF)
    {
      switch (c)
        {
        case 'i':
          inbasename = optarg;
          break;
        case 'o':
          outfd = open (optarg, O_WRONLY | O_CREAT | O_TRUNC, 0666);
          if (outfd < 0)
            {
              perror ("couldn't open output file");
              exit (1);
            }
          break;
        default:
          fprintf (stderr, "Usage:\n");
          fprintf (stderr, "%s [-i input] [-o output]\n", argv[0]);
          fprintf (stderr, "input and output default to stdin/stdout\n");
          exit (1);


        }

    }

  if (!inbasename)
    {
      fprintf (stderr, "Must specify input name\n");
      exit (1);
    }


  sprintf (inname, "%s.%s", inbasename, ext[0]);
  infd = open (inname, O_RDONLY);
  if (infd < 0)
    {
      perror ("couldn't open input file");
      exit (1);
    }

  getfdline (infd, header, sizeof (header));

  if (strcmp (header, "P4\n"))
    {
      fprintf (stderr, "Input must be in PBM format\n");
      exit (-1);
    }

  getfdline (infd, header, sizeof (header));
  w = h = 0;
  sscanf (header, "%d %d", &w, &h);

  inoff = lseek (infd, 0L, SEEK_CUR);

  lseek (infd, 0L, SEEK_END);
  inlength = lseek (infd, 0L, SEEK_CUR);

  close (infd);

  fprintf (stderr,
           "Input: PBM %d by %d pixel %d bytes long\n", w, h, inlength);


  ibpl = ((w + 7) >> 3);

  buf = malloc (ibpl);

  outoff = sprintf (header, "P0\n%d %d\n", w, h);
  write (outfd, header, outoff);

  for (pass = 0; pass < 3; ++pass)
    {

      sprintf (inname, "%s.%s", inbasename, ext[pass]);

      infd = open (inname, O_RDONLY);
      if (infd < 0)
        {
          perror ("couldn't open input file");
          exit (1);
        }

      getfdline (infd, header, sizeof (header));
      getfdline (infd, header, sizeof (header));

      q = h;
      while (q--)
        {
          read (infd, buf, ibpl);
          write (outfd, buf, ibpl);
        }
      close (infd);

    }

  close (outfd);

  return 0;
}
