/*
 * koch.c:
 *
 * Copyright (c) 2001 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] = "$Id: koch.c,v 1.2 2001/05/24 13:56:19 root Exp $";

/*
 * $Log: koch.c,v $
 * Revision 1.2  2001/05/24 13:56:19  root
 * *** empty log message ***
 *
 * Revision 1.1  2001/05/24 13:56:04  root
 * Initial revision
 *
 */

#include <stdio.h>
#include <math.h>

enum
{
  NONE,
  UP,
  LEFT,
  DOWN,
  RIGHT,
};

static int x, y, w, h, sx, sy, obpl;
static unsigned char *in, *out;
static int ox;

static int
log2 (int value)
{
  int result = 0;
  while (value > 1)
    {
      value >>= 1;
      result++;
    }
  return result;
}

#define BLACK (*out)|=ox
#define WHITE (*out)&=~ox

#define SIZE 16                 /* queue size: number of pixels remembered */
#define MAX  16                 /* relative weight of youngest pixel in the
                                 * queue, versus the oldest pixel */

static int weights[SIZE];       /* weights for the errors of recent pixels */

static void
init_weights (int a[], int size, int max)
{
  double m = exp (log (max) / (size - 1));
  double v;
  int i;

  for (i = 0, v = 1.0; i < size; i++)
    {
      a[i] = (int) (v + 0.5);   /* store rounded value */
      v *= m;                   /* next value */
    }                           /*for */
}

static int error[SIZE];         /* queue with error values of recent pixels */

static void
dither ()
{
  int i, pvalue;
  int err = 0;

  for (i = 0; i < SIZE; i++)
    err += error[i] * weights[i];

  pvalue = (*in) + err / MAX;

  if (pvalue < 128)
    {
      pvalue = 0;
      BLACK;
    }
  else
    {
      pvalue = 255;
      WHITE;
    }

  memmove (error, error + 1, (SIZE - 1) * sizeof error[0]);
  error[SIZE - 1] = *in - pvalue;
}


static void
move (int direction)
{
  if (x >= 0 && x < w && y >= 0 && y < h)
    dither ();

  switch (direction)
    {
    case LEFT:
      x--;
      in -= sx;

      ox <<= 1;
      if (ox == 256)
        {
          ox = 1;
          out--;
        }

      break;
    case RIGHT:
      x++;
      in += sx;

      ox >>= 1;
      if (!ox)
        {
          ox = 128;
          out++;
        }
      break;
    case UP:
      y--;
      in -= sy;
      out -= obpl;
      break;
    case DOWN:
      y++;
      in += sy;
      out += obpl;
      break;
    }
}


static void
hilbert (int level, int direction)
{
  if (level == 1)
    {
      switch (direction)
        {
        case LEFT:
          move (RIGHT);
          move (DOWN);
          move (LEFT);
          break;
        case RIGHT:
          move (LEFT);
          move (UP);
          move (RIGHT);
          break;
        case UP:
          move (DOWN);
          move (RIGHT);
          move (UP);
          break;
        case DOWN:
          move (UP);
          move (LEFT);
          move (DOWN);
          break;
        }
      return;

    }
  switch (direction)
    {
    case LEFT:
      hilbert (level - 1, UP);
      move (RIGHT);
      hilbert (level - 1, LEFT);
      move (DOWN);
      hilbert (level - 1, LEFT);
      move (LEFT);
      hilbert (level - 1, DOWN);
      break;
    case RIGHT:
      hilbert (level - 1, DOWN);
      move (LEFT);
      hilbert (level - 1, RIGHT);
      move (UP);
      hilbert (level - 1, RIGHT);
      move (RIGHT);
      hilbert (level - 1, UP);
      break;
    case UP:
      hilbert (level - 1, LEFT);
      move (DOWN);
      hilbert (level - 1, UP);
      move (RIGHT);
      hilbert (level - 1, UP);
      move (UP);
      hilbert (level - 1, RIGHT);
      break;
    case DOWN:
      hilbert (level - 1, RIGHT);
      move (UP);
      hilbert (level - 1, DOWN);
      move (LEFT);
      hilbert (level - 1, DOWN);
      move (DOWN);
      hilbert (level - 1, LEFT);
      break;
    }
}


void
koch_dither (int uw, int uh, unsigned char *uin, int usx, int usy,
             unsigned char *uout, int uobpl)
{
  int level, size;

  in = uin;
  w = uw;
  h = uh;
  sx = usx;
  sy = usy;
  out = uout;
  obpl = uobpl;

  size = w > h ? w : h;
  level = log2 (size);
  if ((1 << level) < size)
    level++;

  x = y = 0;
  init_weights (weights, SIZE, MAX);


  ox = 256;

  if (level > 0)
    hilbert (level, UP);

  move (NONE);
}
