/*
 * pcmto3pbm.c:
 *
 * Copyright (c) 2001 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] =
  "$Id: pcmto3pbm.c,v 1.3 2001/05/25 01:24:37 root Exp $";

/*
 * $Log: pcmto3pbm.c,v $
 * Revision 1.3  2001/05/25 01:24:37  root
 * #
 *
 * Revision 1.2  2001/05/25 01:24:15  root
 * #
 *
 */


#include <malloc.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>

static int
getfdline (int fd, char *buf, int buflen)
{
  char *ptr = buf;

  buflen--;

  while (buflen > 0)
    {

      read (fd, ptr, 1);
      *(ptr + 1) = 0;
      buflen--;

      if (*ptr == '\n')
        {
          if (*buf == '#')
            {
              ptr = buf;
            }
          else
            {
              return 1;
            }
        }
      else
        ptr++;
    }

  return 0;
}


int
main (int argc, char *argv[])
{
  int infd, outfd;
  off_t inoff, outoff;
  size_t inlength, outlength;
  int w, h, ibpl;
  int c, q;
  char *ext[3] = { "red", "green", "blue" };
  int pass;

  unsigned char *buf;
  unsigned char *outbasename = NULL;
  char header[1024];
  char outname[1024];

  infd = 0;
  outfd = 1;
  while ((c = getopt (argc, argv, "i:o:")) != EOF)
    {
      switch (c)
        {
        case 'i':
          infd = open (optarg, O_RDONLY);
          if (infd < 0)
            {
              perror ("couldn't open input file");
              exit (1);
            }
          break;
        case 'o':
          outbasename = optarg;
          break;
        default:
          fprintf (stderr, "Usage:\n");
          fprintf (stderr, "%s [-i input] [-o output]\n", argv[0]);
          fprintf (stderr, "input and output default to stdin/stdout\n");
          exit (1);


        }

    }

  if (!outbasename)
    {
      fprintf (stderr, "Must specify output name\n");
      exit (1);
    }


  getfdline (infd, header, sizeof (header));

  if (strcmp (header, "P0\n"))
    {
      fprintf (stderr, "Input must be in PCM format\n");
      exit (-1);
    }

  getfdline (infd, header, sizeof (header));
  w = h = 0;
  sscanf (header, "%d %d", &w, &h);


  inoff = lseek (infd, 0L, SEEK_CUR);

  lseek (infd, 0L, SEEK_END);
  inlength = lseek (infd, 0L, SEEK_CUR);
  lseek (infd, inoff, SEEK_SET);

  fprintf (stderr,
           "Input: PCM %d by %d pixel %d bytes long\n", w, h, inlength);


  ibpl = ((w + 7) >> 3);

  buf = malloc (ibpl);


  for (pass = 0; pass < 3; ++pass)
    {

      sprintf (outname, "%s.%s", outbasename, ext[pass]);

      outfd = open (outname, O_RDWR | O_CREAT | O_TRUNC, 0666);
      if (outfd < 0)
        {
          perror ("couldn't open output file");
          exit (1);
        }


      outoff = sprintf (header, "P4\n%d %d\n", w, h);
      outlength = outoff + h * ibpl;

      fprintf (stderr, "Output: %s PBM %d by %d pixel %d bytes long\n",
               outname, w, h, outlength);

      write (outfd, header, outoff);

      q = h;
      while (q--)
        {
          read (infd, buf, ibpl);
          write (outfd, buf, ibpl);
        }
      close (outfd);

    }

  close (infd);

  return 0;
}
