/*
 * pcmtoppm.c:
 *
 * Copyright (c) 2001 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] =
  "$Id: pcmtoppm.c,v 1.4 2001/05/25 01:24:37 root Exp $";

/*
 * $Log: pcmtoppm.c,v $
 * Revision 1.4  2001/05/25 01:24:37  root
 * #
 *
 * Revision 1.3  2001/05/25 01:24:15  root
 * #
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>
#include <strings.h>


static void
undither (int w, int h, unsigned char *in, int ibpl, unsigned char *out,
          int sx, int sy)
{
  unsigned char *irow, *orow;
  int ix;
  int x, y;

  y = h;

  while (y--)
    {
      x = w;
      irow = in;
      orow = out;
      ix = 128;

      while (x--)
        {

          if ((*irow) & ix)
            {
              *orow = 0;
            }
          else
            {
              *orow = 255;
            }

          orow += sx;

          ix >>= 1;
          if (!ix)
            {
              irow++;
              ix = 128;
            }


        }
      in += ibpl;
      out += sy;
    }
}






static void
my_bcopy (unsigned char *in, unsigned char *out, int n)
{
  while (n--)
    *(out++) = *(in++);
}


static int
getfdline (int fd, char *buf, int buflen)
{
  char *ptr = buf;

  buflen--;

  while (buflen > 0)
    {

      read (fd, ptr, 1);
      *(ptr + 1) = 0;
      buflen--;

      if (*ptr == '\n')
        {
          if (*buf == '#')
            {
              ptr = buf;
            }
          else
            {
              return 1;
            }
        }
      else
        ptr++;
    }

  return 0;
}


int
main (int argc, char *argv[])
{
  char buf[1024];
  int infd, outfd;
  off_t inoff, outoff;
  size_t inlength, outlength;
  int w, h, ibpl;
  int c;
  int koch = 0;
  int floyd = 0;
  int thresh = 0;

  int pagesize = getpagesize ();


  unsigned char *in, *inbase;
  unsigned char *out, *outbase;

  infd = 0;
  outfd = 1;

  while ((c = getopt (argc, argv, "i:o:")) != EOF)
    {
      switch (c)
        {
        case 'i':
          infd = open (optarg, O_RDONLY);
          if (infd < 0)
            {
              perror ("couldn't open input file");
              exit (1);
            }
          break;
        case 'o':
          outfd = open (optarg, O_RDWR | O_CREAT | O_TRUNC, 0666);
          if (outfd < 0)
            {
              perror ("couldn't open output file");
              exit (1);
            }
          break;
        default:
          fprintf (stderr, "Usage:\n");
          fprintf (stderr, "%s [-i input] [-o output]\n", argv[0]);
          fprintf (stderr, "input and output default to stdin/stdout\n");
          exit (1);


        }

    }


  getfdline (infd, buf, sizeof (buf));

  if (strcmp (buf, "P0\n"))
    {
      fprintf (stderr, "Input must be in PCM format\n");
      exit (-1);
    }

  getfdline (infd, buf, sizeof (buf));
  w = h = 0;
  sscanf (buf, "%d %d", &w, &h);

  inoff = lseek (infd, 0L, SEEK_CUR);

  lseek (infd, 0L, SEEK_END);
  inlength = lseek (infd, 0L, SEEK_CUR);
  lseek (infd, 0L, SEEK_SET);

  fprintf (stderr,
           "Input: PCM %d by %d pixel %d bytes long\n", w, h, inlength);


  inlength += (pagesize - 1);
  inlength &= ~(pagesize - 1);

  inbase = mmap (NULL, inlength, PROT_READ, MAP_SHARED, infd, 0);

  if (inbase == MAP_FAILED)
    {
      fprintf (stderr, "Failed to mmap input file...\n");
      fprintf (stderr, "try using -i option\n");
      exit (1);
    }

  in = inbase + inoff;
  ibpl = ((w + 7) >> 3);

  outoff = sprintf (buf, "P6\n%d %d\n255\n", w, h);
  outlength = outoff + h * w * 3;

  lseek (outfd, outlength - 1, SEEK_SET);
  write (outfd, "", 1);
  lseek (outfd, 0L, SEEK_SET);

  fprintf (stderr, "Output: PPM %d by %d pixel %d bytes long\n",
           w, h, outlength);

  outlength += (pagesize - 1);
  outlength &= ~(pagesize - 1);

  outbase = mmap (NULL, outlength, PROT_WRITE, MAP_SHARED, outfd, 0);

  if (outbase == MAP_FAILED)
    {
      fprintf (stderr, "Failed to mmap output file...\n");
      fprintf (stderr, "try using -o option\n");
      exit (1);
    }

  my_bcopy (buf, outbase, strlen (buf));
  out = outbase + outoff;

  undither (w, h, in, ibpl, out, 3, 3 * w);
  in += ibpl * h;
  out++;
  undither (w, h, in, ibpl, out, 3, 3 * w);
  in += ibpl * h;
  out++;
  undither (w, h, in, ibpl, out, 3, 3 * w);

  munmap (outbase, outlength);
  close (outfd);
  munmap (inbase, inlength);
  close (infd);

  return 0;
}
